"""New Authentication System Command."""
import os
import shutil

from cleo import Command


class AuthCommand(Command):
    """
    Creates an authentication system.

    auth
    """

    def handle(self):
        self.info('Scaffolding Application ...')
        module_path = os.path.dirname(os.path.realpath(__file__))

        with open('routes/web.py', 'a') as f:
            # add all the routes
            f.write("\nfrom masonite.auth import Auth \n")
            f.write("ROUTES += Auth.routes()")
            f.write('\n')

        # move controllers
        shutil.copyfile(module_path + "/../snippets/auth/controllers/LoginController.py",
                        os.getcwd() + "/app/http/controllers/auth/LoginController.py")
        shutil.copyfile(module_path + "/../snippets/auth/controllers/RegisterController.py",
                        os.getcwd() + "/app/http/controllers/auth/RegisterController.py")
        shutil.copyfile(module_path + "/../snippets/auth/controllers/HomeController.py",
                        os.getcwd() + "/app/http/controllers/auth/HomeController.py")
        shutil.copyfile(module_path + "/../snippets/auth/controllers/ConfirmController.py",
                        os.getcwd() + "/app/http/controllers/auth/ConfirmController.py")
        shutil.copyfile(module_path + "/../snippets/auth/controllers/PasswordController.py",
                        os.getcwd() + "/app/http/controllers/auth/PasswordController.py")
        # move templates
        shutil.copytree(module_path + "/../snippets/auth/templates/auth",
                        os.getcwd() + "/resources/templates/auth")

        self.info('Project Scaffolded. You now have 5 new controllers, 7 new templates and 9 new routes')



"""Remove Preset"""
import os
import shutil
from ..presets import Preset


class Remove(Preset):
    """Removes any defined Preset"""

    def install(self):
        """Install the preset"""
        self.update_packages()
        self.update_bootstrapping()
        self.update_webpack_configuration()
        self.remove_node_modules()
        if os.path.exists(os.path.realpath('resources/sass/_variables.scss')):
            os.remove('resources/sass/_variables.scss')
        shutil.rmtree('resources/js/components', ignore_errors=True)
        shutil.rmtree('public/css', ignore_errors=True)
        shutil.rmtree('public/js', ignore_errors=True)

    def update_package_array(self, packages={}):
        """Updates the packages array to remove React, VueJS, and Bootstrap packages"""
        packages_to_remove = [
            'bootstrap',
            'jquery',
            'popper.js',
            'vue',
            'vue-template-compiler',
            '@babel/preset-react',
            'react',
            'react-dom'
        ]
        for package in packages_to_remove:
            packages.pop(package, None)

        return packages

    def update_webpack_configuration(self):
        """Copy webpack.mix.js file into application"""
        shutil.copyfile(os.path.dirname(__file__) + '/remove-stubs/webpack.mix.js', 'webpack.mix.js')

    def update_bootstrapping(self):
        """Copies template app.js file into application"""
        for directory in ['resources/sass', 'resources/js']:
            if not os.path.exists(os.path.realpath(directory)):
                os.makedirs(os.path.realpath(directory))
        with open('resources/sass/app.scss', 'w') as f:
            f.write('')
        shutil.copyfile(os.path.dirname(__file__) + '/remove-stubs/app.js', 'resources/js/app.js')
        shutil.copyfile(os.path.dirname(__file__) + '/remove-stubs/bootstrap.js', 'resources/js/bootstrap.js')



"""New Preset System Command."""
from cleo import Command
from ..commands.presets.React import React
from ..commands.presets.Vue import Vue
from ..commands.presets.Bootstrap import Bootstrap
from ..commands.presets.Remove import Remove


class PresetCommand(Command):
    """
    Swap the front-end scaffolding for the application

    preset
        {name : Name of the preset}
    """

    def handle(self):
        self.info('Scaffolding Application ...')
        preset_name = self.argument('name')
        if preset_name not in ['react', 'vue', 'remove', 'bootstrap']:
            raise ValueError('Invalid preset')
        return getattr(self, preset_name)()

    def remove(self):
        """Removes frontend scaffolding"""
        Remove().install()
        self.info('Frontend scaffolding removed successfully.')

    def react(self):
        """Add React frontend while also removing Vue (if it was previously selected)"""
        React().install()
        self.info('React scaffolding installed successfully.')
        self.comment('Please run "npm install && npm run dev" to compile your fresh scaffolding.')

    def vue(self):
        """Add Vue frontend while also removing React (if it was previously selected)"""
        Vue().install()
        self.info('Vue scaffolding installed successfully.')
        self.comment('Please run "npm install && npm run dev" to compile your fresh scaffolding.')

    def bootstrap(self):
        """Add Bootstrap Sass scafolding"""
        Bootstrap().install()
        self.info('Bootstrap scaffolding installed successfully.')
        self.comment('Please run "npm install && npm run dev" to compile your fresh scaffolding.')



"""React Preset"""
import os
import shutil
from ..presets import Preset


class React(Preset):
    """
    Configure the front-end scaffolding for the application to use React

    Will also remove VueJS as React and Vue are a bit mutally exclusive
    """

    def install(self):
        """Install the preset"""
        self.ensure_component_directory_exists()
        self.update_packages()
        self.update_webpack_configuration()
        self.update_bootstrapping()
        self.update_component()
        self.remove_node_modules()

    def update_package_array(self, packages={}):
        """
        Updates the packages array to include React specific packages
        but also remove VueJS ones
        """
        for package in ['vue', 'vue-template-compiler']:
            packages.pop(package, None)

        packages['@babel/preset-react'] = '^7.0.0'
        packages['react'] = '^16.2.0'
        packages['react-dom'] = '^16.2.0'
        return packages

    def update_webpack_configuration(self):
        """Copy webpack.mix.js file into application"""
        shutil.copyfile(os.path.dirname(__file__) + '/react-stubs/webpack.mix.js', 'webpack.mix.js')

    def update_component(self):
        """
        Copy example React component into application
        (delete example Vue component if it exists)
        """
        vue_component = 'resources/js/components/ExampleComponent.vue'
        if os.path.exists(os.path.realpath(vue_component)):
            os.remove(vue_component)
        shutil.copyfile(os.path.dirname(__file__) + '/react-stubs/Example.js', 'resources/js/components/Example.js')

    def update_bootstrapping(self):
        """Copies template app.js file into application"""
        shutil.copyfile(os.path.dirname(__file__) + '/react-stubs/app.js', 'resources/js/app.js')



"""Authentication Class."""


class Auth:
    """Facade class for the Guard class"""

    @staticmethod
    def routes():
        from ..routes import Get, Post
        return [
            Get().route('/login', 'auth.LoginController@show').name('login'),
            Get().route('/logout', 'auth.LoginController@logout').name('logout'),
            Post().route('/login', 'auth.LoginController@store'),
            Get().route('/register', 'auth.RegisterController@show').name('register'),
            Post().route('/register', 'auth.RegisterController@store'),
            Get().route('/home', 'auth.HomeController@show').name('home'),
            Get().route('/email/verify', 'auth.ConfirmController@verify_show').name('verify'),
            Get().route('/email/verify/@id:signed', 'auth.ConfirmController@confirm_email'),
            Get().route('/email/verify/@id:signed', 'auth.ConfirmController@confirm_email'),
            Get().route('/password', 'auth.PasswordController@forget').name('forgot.password'),
            Post().route('/password', 'auth.PasswordController@send'),
            Get().route('/password/@token/reset', 'auth.PasswordController@reset').name('password.reset'),
            Post().route('/password/@token/reset', 'auth.PasswordController@update'),
        ]



"""An Authentication Service Provider."""

from ..auth.guards import Guard, WebGuard
from ..auth import Auth
from ..helpers import config
from ..provider import ServiceProvider


class AuthenticationProvider(ServiceProvider):

    wsgi = False

    def register(self):
        guard = Guard(self.app)
        guard.register_guard('web', WebGuard)
        self.app.simple(guard)
        self.app.swap(Auth, guard)

    def boot(self, auth: Auth):
        auth.set(config('auth.auth.defaults.guard'))



import os
import shutil
import json


class Preset:

    def ensure_component_directory_exists(self):
        """Ensure the component directories we need exist."""
        directory = 'resources/js/components'
        if not os.path.exists(os.path.realpath(directory)):
            os.makedirs(os.path.realpath(directory))

    def update_packages(self, dev=True):
        """Update the "package.json" file."""
        if not os.path.exists(os.path.realpath('package.json')):
            return

        configuration_key = 'devDependencies' if dev else 'dependencies'

        packages = {}
        with open(os.path.realpath('package.json'), 'r+') as f:
            packages = json.load(f)
            packages[configuration_key] = self.update_package_array(
                packages[configuration_key] if configuration_key in packages else {}
            )
            f.seek(0)  # Rewind to beginning of file
            f.truncate()
            f.write(
                json.dumps(packages, sort_keys=True, indent=4)
            )

    def remove_node_modules(self):
        """Remove the installed Node modules."""
        for filename in ['package-lock.json', 'yarn.lock']:
            if os.path.exists(os.path.realpath(filename)):
                os.remove(filename)
        shutil.rmtree('node_modules', ignore_errors=True)



from ..app import App
import copy


def create_container():
    container = copy.deepcopy(App())
    from .generate_wsgi import generate_wsgi
    from config import providers

    container.bind('WSGI', generate_wsgi())
    container.bind('Container', container)

    # container.bind('ProvidersConfig', providers)
    container.bind('Providers', [])
    container.bind('WSGIProviders', [])

    """Bind all service providers
        Let's register everything into the Service Container. Once everything is
        in the container we can run through all the boot methods. For reasons
        some providers don't need to execute with every request and should
        only run once when the server is started. Providers will be ran
        once if the wsgi attribute on a provider is False.
        """

    for provider in providers.PROVIDERS:
        located_provider = provider()
        located_provider.load_app(container).register()
        if located_provider.wsgi:
            container.make('WSGIProviders').append(located_provider)
        else:
            container.make('Providers').append(located_provider)

    for provider in container.make('Providers'):
        container.resolve(provider.boot)

    """Get the application from the container
        Some providers may change the WSGI Server like wrapping the WSGI server
        in a Whitenoise container for an example. Let's get a WSGI instance
        from the container and pass it to the application variable. This
        will allow WSGI servers to pick it up from the command line
        """

    return container



"""The Masonite Response Object."""

import json
import mimetypes
from pathlib import Path

from orator import LengthAwarePaginator, Model, Paginator
from orator.support.collection import Collection

from .app import App
from .exceptions import ResponseError
from .helpers.Extendable import Extendable


class Response(Extendable):
    """A Response object to be used to abstract the logic of getting a response ready to be returned.

    Arguments:
        app {masonite.app.App} -- The Masonite container.
    """

    def __init__(self, app: App):
        self.app = app
        self.request = self.app.make('Request')

    def json(self, payload, status=200):
        """Gets the response ready for a JSON response.

        Arguments:
            payload {dict|list} -- Either a dictionary or a list.

        Returns:
            string -- Returns a string representation of the data
        """
        self.app.bind('Response', bytes(json.dumps(payload), 'utf-8'))
        self.make_headers(content_type="application/json; charset=utf-8")
        self.request.status(status)

        return self.data()

    def paginated_json(self, paginator, status=200):
        """Determine type of paginated instance and return JSON response.

        Arguments:
            paginator {Paginator|LengthAwarePaginator} --
                Either an Orator Paginator or LengthAwarePaginator object

        Returns:
            string -- Returns a string representation of the data
        """
        # configured param types
        page_size_parameter = 'page_size'
        page_parameter = 'page'

        # try to capture request input for page_size and/or page
        page_size_input = self.request.input(page_size_parameter)
        page_input = self.request.input(page_parameter)
        try:
            page_size = (
                int(page_size_input)
                if page_size_input and int(page_size_input) > 0
                else paginator.per_page
            )
        except ValueError:
            page_size = paginator.per_page

        try:
            page = (
                int(page_input)
                if page_input and int(page_input) > 0
                else paginator.current_page
            )
        except ValueError:
            page = paginator.current_page

        # don't waste time instantiating new paginator if no change
        payload = {
            'total': (
                paginator.total
                if isinstance(paginator, LengthAwarePaginator)
                else None
            ),
            'count': paginator.count(),
            'per_page': page_size,
            'current_page': page,
            'last_page': (
                paginator.last_page
                if isinstance(paginator, LengthAwarePaginator)
                else None
            ),
            'from': (page_size * (page - 1)) + 1,
            'to': page_size * page,
            'data': paginator.serialize()
        }

        # remove fields not relevant to Paginator instance
        if isinstance(paginator, Paginator):
            del payload['total']
            del payload['last_page']

        return self.json(payload, status)

    def make_headers(self, content_type="text/html; charset=utf-8"):
        """Make the appropriate headers based on changes made in controllers or middleware.

        Keyword Arguments:
            content_type {str} -- The content type to set. (default: {"text/html; charset=utf-8"})
        """
        self.request.header('Content-Length', str(len(self.to_bytes())))

        # If the user did not change it directly
        if not self.request.has_raw_header('Content-Type'):
            self.request.header('Content-Type', content_type)

    def data(self):
        """Get the data that will be returned to the WSGI server.

        Returns:
            string -- Returns a string representation of the response
        """
        if self.app.has('Response'):
            return self.app.make('Response')

        return ''

    def converted_data(self):
        """Converts the data appropriately so the WSGI server can handle it.

        Returns:
            string -- Returns a string representation of the data
        """
        if isinstance(self.data(), (dict, list)):
            return json.dumps(self.data())
        else:
            return self.data()

    def view(self, view, status=200):
        """Set a string or view to be returned.

        Arguments:
            view {string|dict|list|masonite.view.View} -- Some data type that is an appropriate response.

        Keyword Arguments:
            status {int} -- The Response status code. (default: {200})

        Raises:
            ResponseError -- If a data type that is not an acceptable response type is returned.

        Returns:
            string|dict|list -- Returns the data to be returned.
        """
        if not self.request.get_status():
            self.request.status(status)

        if isinstance(view, (dict, list)):
            return self.json(view, status=self.request.get_status())
        elif isinstance(view, (Collection, Model)):
            return self.json(view.serialize(), status=self.request.get_status())
        elif isinstance(view, int):
            view = str(view)
        elif isinstance(view, Responsable):
            view = view.get_response()
        elif isinstance(view, self.request.__class__):
            view = self.data()
        if isinstance(view, (Paginator, LengthAwarePaginator)):
            return self.paginated_json(view, status=self.request.get_status())
        elif view is None:
            raise ResponseError('Responses cannot be of type: None. Did you return anything in your responsable method?')

        if isinstance(view, str):
            self.app.bind('Response', bytes(view, 'utf-8'))
        else:
            self.app.bind('Response', view)

        self.make_headers()

        return self.data()

    def redirect(self, location=None, status=302):
        """Set the redirection on the server.

        Keyword Arguments:
            location {string} -- The URL to redirect to (default: {None})
            status {int} -- The Response status code. (default: {302})

        Returns:
            string -- Returns the data to be returned.
        """
        self.request.status(status)
        if not location:
            location = self.request.redirect_url

        self.request.reset_headers()
        self.request.header('Location', location)
        self.view('Redirecting ...')

        return self.data()

    def to_bytes(self):
        """Converts the data to bytes so the WSGI server can handle it.

        Returns:
            bytes -- The converted response to bytes.
        """
        return self.converted_data()


class Responsable:

    def get_response(self):
        raise NotImplementedError("This class does not implement a 'get_response()' method")


class Download(Responsable):
    """Download class to help show files in the browser or force
        a download for the client browser.

    Arguments:
        location {string} -- The path you want to download.

    Keyword Arguments:
        force {bool} -- Whether you want the client's browser to force the file download (default: {False})
        name {str} -- The name you want the file to be called when downloaded (default: {'profile.jpg'})
    """

    def __init__(self, location, force=False, name='1'):
        self.location = location
        self._force = force
        self.name = name
        self.container = None

    def force(self):
        """Sets the force option.

        Returns:
            self
        """
        self._force = True
        return self

    def get_response(self):
        """Handles the way the response should be handled by the server.

        Returns:
            bytes - Returns bytes required for the server to handle the download.
        """
        if not self.container:
            from wsgi import container
            self.container = container

        request = self.container.make('Request')

        with open(self.location, 'rb') as filelike:
            data = filelike.read()

        if self._force:
            request.header('Content-Type', 'application/octet-stream')
            request.header('Content-Disposition', 'attachment; filename="{}{}"'.format(self.name, self.extension(self.location)))
        else:
            request.header('Content-Type', self.mimetype(self.location))

        return data

    def mimetype(self, path):
        """Gets the mimetime of a path

        Arguments:
            path {string} -- The path of the file to download.

        Returns:
            string -- The mimetype for use in headers
        """
        return mimetypes.guess_type(path)[0]

    def extension(self, path):
        return Path(path).suffix




import io


def generate_wsgi():
    return {
        'wsgi.version': (1, 0),
        'wsgi.multithread': False,
        'wsgi.multiprocess': True,
        'wsgi.run_once': False,
        'wsgi.input': io.BytesIO(),
        'SERVER_SOFTWARE': 'gunicorn/19.7.1',
        'REQUEST_METHOD': 'GET',
        'QUERY_STRING': 'application=Masonite',
        'RAW_URI': '/',
        'SERVER_PROTOCOL': 'HTTP/1.1',
        'HTTP_HOST': '127.0.0.1:8000',
        'HTTP_ACCEPT': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
        'HTTP_UPGRADE_INSECURE_REQUESTS': '1',
        'HTTP_COOKIE': 'setcookie=value',
        'HTTP_USER_AGENT': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_2) AppleWebKit/604.4.7 (KHTML, like Gecko) Version/11.0.2 Safari/604.4.7',
        'HTTP_ACCEPT_LANGUAGE': 'en-us',
        'HTTP_ACCEPT_ENCODING': 'gzip, deflate',
        'HTTP_CONNECTION': 'keep-alive',
        'wsgi.url_scheme': 'http',
        'REMOTE_ADDR': '127.0.0.1',
        'REMOTE_PORT': '62241',
        'SERVER_NAME': '127.0.0.1',
        'SERVER_PORT': '8000',
        'PATH_INFO': '/',
        'SCRIPT_NAME': ''
    }




class AuthenticationGuard:

    def guard(self, guard):
        """Specify the guard you want to use

        Arguments:
            guard {[type]} -- [description]
        """
        from .Guard import Guard
        return Guard(self.app).make(guard)

    def register_guard(self, key, cls=None):
        """Registers a new guard class.

        Arguments:
            key {string|dict} -- The key to name the guard to a dictionary of key: values

        Keyword Arguments:
            cls {object} -- A guard class. (default: {None})

        Returns:
            None
        """
        from .Guard import Guard
        if isinstance(key, dict):
            return Guard.guards.update(key)

        return Guard.guards.update({key: cls})

    def register_driver(self, key, cls):
        """Registers a new driver with the current guard class.

        Arguments:
            key {string} -- The key to register the driver to.
            cls {class} -- A guard class.
        """
        self.drivers.update({key: cls})

    def make(self, key):
        """Makes a new driver from the current guard class.

        Arguments:
            key {string} -- The key to for the driver to make.

        Raises:
            DriverNotFound: Thrown when the driver is not registered.

        Returns:
            object -- Returns a guard driver object.
        """
        if key in self.drivers:
            self.driver = self.app.resolve(self.drivers[key])
            return self.driver

        raise DriverNotFound("Could not find the driver {}".format(key))



"""Module for the ache disk driver."""

import glob
import os
import time

from ...contracts import CacheContract
from ...drivers import BaseCacheDriver
from ...helpers import config


class CacheDiskDriver(CacheContract, BaseCacheDriver):
    """Class for the cache disk driver."""

    def __init__(self):
        """Cache disk driver constructor.

        Arguments:
            CacheConfig {config.cache} -- Cache configuration module.
            Application {config.application} -- Application configuration module.
        """
        self.config = config('cache')
        self.appconfig = config('application')
        self.cache_forever = None

    def store(self, key, value, extension=".txt", location=None):
        """Store content in cache file.

        Arguments:
            key {string} -- The key to store the cache file into
            value {string} -- The value you want to store in the cache

        Keyword Arguments:
            extension {string} -- the extension you want to append to the file (default: {".txt"})
            location {string} -- The path you want to store the cache into. (default: {None})

        Returns:
            string -- Returns the key
        """
        self.cache_forever = True
        if not location:
            location = self.config.DRIVERS['disk']['location']

        location += '/'
        path = os.path.join(location, key + extension)
        if not os.path.exists(path):
            self._create_directory(path)

        with open(path, 'w') as file:
            file.write(value)

        return key

    def store_for(self, key, value, cache_time, cache_type, extension=".txt", location=None):
        """Store the cache for a specific amount of time.

        Arguments:
            key {string} -- The key to store the cache file into
            value {string} -- The value you want to store in the cache
            cache_time {int|string} -- The time as a string or an integer (1, 2, 5, 100, etc)
            cache_type {string} -- The type of time to store for (minute, minutes, hours, seconds, etc)

        Keyword Arguments:
            extension {string} -- the extension you want to append to the file (default: {".txt"})
            location {string} -- The path you want to store the cache into. (default: {None})

        Raises:
            ValueError -- Thrown if an invalid cache type was caught (like houes instead of hours).

        Returns:
            string -- Returns the key
        """
        self.cache_forever = False

        cache_for_time = self.calculate_time(cache_type, cache_time)

        cache_for_time = cache_for_time + time.time()

        key = self.store(
            key + ":" + str(cache_for_time),
            value, extension, location
        )

        return key

    def get(self, key):
        """Get the data from a key in the cache."""
        if not self.is_valid(key):
            return None

        cache_path = self.config.DRIVERS['disk']['location'] + "/"
        content = ""

        if self.cache_forever:
            glob_path = cache_path + key + '*'
        else:
            glob_path = cache_path + key + ':*'

        try:
            with open(glob.glob(glob_path)[0], 'r') as file:
                content = file.read()
        except IndexError:
            pass

        return content

    def delete(self, key):
        """Delete file cache."""
        cache_path = self.config.DRIVERS['disk']['location'] + "/"
        if self.cache_forever:
            glob_path = cache_path + key + '*'
        else:
            glob_path = cache_path + key + ':*'

        for template in glob.glob(glob_path):
            os.remove(template)

    def update(self, key, value, location=None):
        """Update a specific cache by key."""
        if not location:
            location = self.config.DRIVERS['disk']['location'] + "/"

        location = os.path.join(location, key)
        cache = glob.glob(location + ':*')[0]

        with open(cache, 'w') as file:
            file.write(str(value))

        return key

    def exists(self, key):
        """Check if the cache exists."""
        cache_path = self.config.DRIVERS['disk']['location'] + "/"
        if self.cache_forever:
            glob_path = cache_path + key + '*'
        else:
            glob_path = cache_path + key + ':*'

        find_template = glob.glob(glob_path)
        if find_template:
            return True
        return False

    def is_valid(self, key):
        """Check if a valid cache."""
        cache_path = self.config.DRIVERS['disk']['location'] + "/"
        if self.cache_forever:
            glob_path = cache_path + key + '*'
        else:
            glob_path = cache_path + key + ':*'

        cache_file = glob.glob(glob_path)
        if cache_file:
            try:
                cache_timestamp = float(
                    os.path.splitext(cache_file[0])[0].split(':')[1]
                )
            except IndexError:
                if self.cache_forever:
                    return True
                else:
                    return False

            if cache_timestamp > time.time():
                return True

        self.delete(key)
        return False

    def _create_directory(self, directory):
        """Creates a new dir.

        Arguments:
            directory {string} -- name of directory to create.

        Returns:
            bool - Returns a boolean if the directory was created.
        """
        if not os.path.exists(os.path.dirname(directory)):
            # Create the path to the model if it does not exist
            os.makedirs(os.path.dirname(directory))
            return True
        return False



"""CSRF Middleware."""

from ..auth import Auth
from ..helpers import config


class GuardMiddleware:
    """Middleware to switch the guard"""

    def __init__(self, auth: Auth):
        self.auth = auth

    def before(self, guard):
        """Sets specified guard for the request.

        Arguments:
            guard {string} -- The key of the guard to set.
        """
        self.auth.set(guard)

    def after(self, _):
        """Sets the default guard back after the request.

        Arguments:
            _ {ignored} -- ignored
        """
        self.auth.set(config('auth.auth.defaults.guard'))



"""Bootstrap Preset"""
import os
import shutil
from ..presets import Preset


class Bootstrap(Preset):
    """Configure the front-end scaffolding for the application to use Bootstrap"""

    def install(self):
        """Install the preset"""
        self.update_packages()
        self.update_sass()
        self.remove_node_modules()

    def update_package_array(self, packages={}):
        """Updates the packages array to include Bootstrap specific packages"""
        packages['bootstrap'] = '^4.0.0'
        packages['jquery'] = '^3.2'
        packages['popper.js'] = '^1.12'
        return packages

    def update_sass(self):
        """Copies Bootstrap scss files into application"""
        directory = 'resources/sass'
        if not os.path.exists(os.path.realpath(directory)):
            os.makedirs(os.path.realpath(directory))
        shutil.copyfile(os.path.dirname(__file__) + '/bootstrap-stubs/_variables.scss', 'resources/sass/_variables.scss')
        shutil.copyfile(os.path.dirname(__file__) + '/bootstrap-stubs/app.scss', 'resources/sass/app.scss')



"""A Guard Class Module."""
from ...app import App
from ...exceptions import DriverNotFound


class Guard:

    guards = {}

    def __init__(self, app: App):
        """Guard Initializer

        Arguments:
            app {masonite.app.App} -- The Masonite container
        """
        self.app = app

    def make(self, key):
        """Makes a guard that has been previously registered

        Arguments:
            key {string} -- The key of the guard to fetch.

        Raises:
            DriverNotFound: Raised when trying to fetch a guard that has not been registered yet.

        Returns:
            [type] -- [description]
        """
        if key in self.guards:
            self._guard = self.app.resolve(self.guards[key])
            return self._guard

        raise DriverNotFound("Could not find the guard: '{}'".format(key))

    def guard(self, key):
        """Alias for the make method.

        Arguments:
            key {string} -- The key of the guard to fetch.

        Returns:
            masonite.guards.* -- An instance of a guard class.
        """
        return self.make(key)

    def set(self, key):
        """Sets the specified guard as the default guard to use.

        Arguments:
            key {string} -- The key of the guard to set.

        Returns:
            masonite.guards.* -- An instance of guard class.
        """
        return self.make(key)

    def get(self):
        """Gets the guard current class.

        Returns:
            masonite.guards.* -- An instance of guard class.
        """
        return self._guard

    def driver(self, key):
        """Gets the driver for the currently set guard class.

        Arguments:
            key {string} -- The key of the driver for the guard to get.

        Returns:
            masonite.drivers.auth.* -- An auth driver class.
        """
        return self._guard.make(key)

    def register_guard(self, key, cls=None):
        """Registers a new guard class.

        Arguments:
            key {string|dict} -- The key to name the guard to a dictionary of key: values

        Keyword Arguments:
            cls {object} -- A guard class. (default: {None})

        Returns:
            None
        """
        if isinstance(key, dict):
            return self.guards.update(key)

        return self.guards.update({key: cls})

    def login(self, *args, **kwargs):
        """Wrapper method to call the guard class method.

        Returns:
            * -- Returns what the guard class method returns.
        """
        return self._guard.login(*args, **kwargs)

    def user(self, *args, **kwargs):
        """Wrapper method to call the guard class method.

        Returns:
            * -- Returns what the guard class method returns.
        """
        return self._guard.user(*args, **kwargs)

    def register(self, *args, **kwargs):
        """Wrapper method to call the guard class method.

        Returns:
            * -- Returns what the guard class method returns.
        """
        return self._guard.register(*args, **kwargs)

    def __getattr__(self, key, *args, **kwargs):
        """Wrapper method to call the guard class methods.

        Returns:
            * -- Returns what the guard class methods returns.
        """
        return getattr(self._guard, key)
